/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "thread_base.hh"
#include <signal.h>
#include <stdexcept>
#include <iostream>

using namespace sends;
using namespace std;

//======================================  Construct a thread base
void*
thread_base::launch(void* p) {
    return reinterpret_cast<thread_base*>(p)->thread_stub();
}

//======================================  Construct a thread base
thread_base::thread_base(void) 
    : mRun(false), mBusy(false)
{
}

//======================================  Destroy a thread
thread_base::~thread_base(void) {
    kill_thread(SIGKILL);
}

//======================================  Issue a signal to the specified thread
int
thread_base::kill_thread(int sig) {
    return pthread_kill(mThreadID, sig);
}

//======================================  Run the thread
int
thread_base::start_thread(void) {
    //----------------------------------  Specify the thread attributes
    pthread_attr_t tattr;
    if (pthread_attr_init (&tattr) == 0) {
	pthread_attr_setdetachstate (&tattr, PTHREAD_CREATE_DETACHED);
    } else {
	throw runtime_error("Error initializing thread attributes");
    }

    //----------------------------------  Create the thread
    int err = pthread_create(&mThreadID, &tattr, &launch, this);

    //----------------------------------  Clean up
    pthread_attr_destroy(&tattr);
    return err;
}

//======================================  Run the thread
int
thread_base::stop_thread(void) {
    mRun  = false;
    return 0;
}

//======================================  Stub function
void*
thread_base::thread_stub(void) {
    mRun  = true;
    mBusy = true;
    void* err = 0;
    try {
	err = thread_entry();
    } catch (exception& e) {
	cerr << "thread_base: Caught exception (" << e.what() 
	     << ") in thread_entry." << endl;
    }
    mBusy = false;
    return err;
}
