/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "PSD.hh"
#include "DFT.hh"
#include "DVecType.hh"

using namespace containers;

//======================================  PSD default constructor
PSD::PSD(void) {
}

//======================================  PSD data constructor
PSD::PSD(const DFT& fs) 
    : fSeries(fs)
{
    if (empty()) return;
    size_type N = size();
    DVecType<double> dv(N);
    if (fs.refDVect().C_data() ) {
	const float* p = reinterpret_cast<const float*>(fs.refDVect().refData());
	for (size_type i=0; i<N; ++i) {
	    register double x = double(*p++);
	    dv[i] = x * x;
	    x = double(*p++);
	    dv[i] += x * x;
	}
    } else if (fs.refDVect().W_data() ) {
	const double* p = reinterpret_cast<const double*>(fs.refDVect().refData());
	for (size_type i=0; i<N; ++i) {
	    register double x = *p++;
	    dv[i] = x * x;
	    x = *p++;
	    dv[i] += x * x;
	}
    } else {
	dv = fs.refDVect();
	dv *= dv;
    }
    setData(getLowFreq(), getFStep(), dv);
}

//======================================  PSD destructor
PSD::~PSD(void) {
}


//======================================  Clone PSD
PSD* 
PSD::clone(void) const {
    return new PSD(*this);
}

//======================================  Fold PSD
void
PSD::fold(void) {
    bool was_double = double_sided();
    fSeries::fold();
    if ( was_double ) *this *= 2;
}
