/*
 *    DMTifc.h  provides a C callable interface for those wishing to
 *    use C code in the data monitor. THe hooks will have to be added
 *    to make any functionality availeble in the C++ class.
 */
#include <stdio.h>
#include "DMTifc.h"

/**
  *   Monitor Initialization function.
  *   DMT_Initialize is a user supplied function to perform monitor
  *   specific initialization.
  */
void DMT_Initialize(int argc, const char *argv[]) {
    printf("Monitor initialization in progress.\n");
}

/**
  *   Monitor Shut down.
  *   DMT_Halt is a user supplied function to perform monitor specific
  *   end-of-task processing.
  */
void DMT_Halt(void) {
    printf("Monitor has completed\n");
}

/**
  *   Process a data frame.
  *   DMT_Frame if a user supplied function to perform monitor specific 
  *   processing of one data frame. A pointer to the frame to be processed 
  *   is passed to DMT_Frame by the main loop routine.
  */
void DMT_Frame(const void* frame) {
    if (Boundary() && getNFrames() !=1) 
        printf("============>  Discontinuity %i\n", getNContig()-1);
    printf("Do Frame %i.\n", getNFrames());
}
