/* -*- mode: c++; c-basic-offset: 4; -*- */
//
//    File: dvTest
//
//    Bit Test monitor class definition.
//
#include "DMTBase.hh"
#include "Time.hh"
#include "TrigClient.hh"
#include "framecpp/Types.hh"
#include <list>
#include <iosfwd>
#include <string>

class ChanDV;
class Time;

/**  @name dvTest
  *  dvTest prints out times (resolutiuon 1 s) with dataValid on a 
  *  specified channel.
  *
  *  The dvTest command line syntax is as follows:
  *
  *  \begin{verbatim}
         dvTest [-conf <cfile>] [-log] [-trig]
     \end{verbatim}
  *
  *  Where:
  *  \begin{tabular}{ll}
  *  -conf <cfile> & Specify the configuration file name [dvTest.cfg]. \\
  *  -log          & Indicates that dvTest is to log all errors found. \\
  *  -trig         & Enable trigger generation.
  *  \end{tabular}
  *
  *  @memo Look for dataValid errors in frames.
  *  @author J. Zweizig 
  *  @version 1.0; Modified November 14, 2006
  */
//@{
//@}

//======================================  Monitor class definition
class dvTest : public DMTBase {
public:
    dvTest(int argc, const char *argv[]);
    ~dvTest();

    //----------------------------------  Print channel status information
    void PrintStatus(std::ostream& out) const;

    void ProcessFrame(const FrameCPP::FrameH* frame);
    bool ReadConfig(void);

private:
    std::string mConfig;
    std::string mStatusFile;
    typedef std::list<ChanDV> ChanList;
    typedef ChanList::iterator channel_iter;
    typedef ChanList::const_iterator const_chan_iter;
    ChanList mAdcChan;
    ChanList mProcChan;
    TrigClient mTrig;
    bool mGblLog;
    bool mGblTrig;
    Time mLastPrint;
};

//
//    ChanBit Class
//
//    The ChanBit class carries the status of each channel to be followed
//    by the bit tester.
//
class ChanDV {
public:
    ChanDV(const char *chan, TrigClient& tc);
    ~ChanDV(void);

    //----------------------------------  Print status page.
    static void PrintHeader(std::ostream& out);
    void PrintStatus(std::ostream& out) const;

    //----------------------------------  Scan the data
    void Scan(const FrameCPP::FrAdcData *data,  const Time& t0);
    void Scan(const FrameCPP::FrProcData *data, const Time& t0);
    void Scan(const FrameCPP::FrVect    *data, const Time& t0);
    void segment(const Time& t0, Interval dT, short val);
    const char* getChannel(void) const;
    void setTrigEnable(bool te);
    void setLog(bool le);
    void writeSegment(void);

public:
    std::string mChanName;
    bool mTrigEnable;
    bool mLogEnable;
    TrigClient& mTrig;
    short  mCurVal;
    Time   mStart;
    Time   mCurrent;

    //-----------------------------------  Statistics.
    long mCount;
    Time mLast;
};

inline const char*
ChanDV::getChannel(void) const {
    return mChanName.c_str();
}
